# frozen_string_literal: true

# yadecli cli
module Yade
  module Common
    module Client
      # base client
      class BaseClient
        # initialize
        # prepare options hash with authorization header for httpparty requests
        def initialize(microservice, resource_name, model_class)
          @microservice = microservice
          @resource_name = resource_name
          @model_class = model_class

          @authentication_client = Yade::Common::Client::AuthenticationClient.new
          @options = { headers: { Authorization: "Bearer #{@authentication_client.access_token}", 'Content-Type' => 'application/json' } }
        end

        # list all
        def list
          response = self.class.get(base_path, @options)

          response.parsed_response.map do |p|
            create_instance.new(p)
          end
        end

        # get by id
        def get(id)
          response = get_request("#{id}")

          if response.success?
            create_instance.new(response.parsed_response)
          else
            raise "Unable to make get request. Response code was #{response.code}"
          end
        end

        private

        def get_request(path = nil)
          self.class.get(full_path(path), @options)
        end

        def full_path(path = nil)
          full_path = base_path
          full_path += "/#{path}" if path != nil

          full_path
        end

        def base_path
          if @resource_name.end_with? 's'
            "/#{@microservice}/api/#{@resource_name}"
          else
            "/#{@microservice}/api/#{@resource_name}s"
          end
        end

        def create_instance
          Object.const_get(@model_class)
        end
      end
    end
  end
end